//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * JvZwb_
 * @author Junpee
 */

#ifndef CAPSULE_H_
#define CAPSULE_H_

#include <Core/Primitive/Vector3.h>
#include <Core/Primitive/Matrix33.h>
#include <Core/Primitive/Matrix34.h>
#include <Core/Primitive/Matrix44.h>

namespace Lamp{

class AxisAlignedBox;
class Cone;
class Line;
class OrientedBox;
class Plane;
class Ray;
class Segment;
class Sphere;
class Triangle;

//------------------------------------------------------------------------------
/**
 * JvZ
 *
 * ̃NX͌pȂŉB
 */
class Capsule{
public:
	//--------------------------------------------------------------------------
	// 萔
	//--------------------------------------------------------------------------
	/// [JvZ
	static const Capsule zero;

	//--------------------------------------------------------------------------
	// RXgN^
	//--------------------------------------------------------------------------
	/**
	 * RXgN^
	 *
	 * ̃RXgN^͏l̐ݒsȂߒl͕słB
	 */
	Capsule(){}

	/**
	 * RXgN^
	 * @param origin _̏l
	 * @param direction ̏l
	 * @param radius ȁl
	 */
	inline Capsule(
		const Vector3& origin, const Vector3& direction, float radius) :
		origin_(origin), direction_(direction), radius_(radius){
	}

	/**
	 * RXgN^
	 * @param originX _X̏l
	 * @param originY _Y̏l
	 * @param originZ _Z̏l
	 * @param directionX X̏l
	 * @param directionY Y̏l
	 * @param directionZ Z̏l
	 * @param radius ȁl
	 */
	inline Capsule(float originX, float originY, float originZ,
		float directionX, float directionY, float directionZ, float radius) :
		origin_(originX, originY, originZ),
		direction_(directionX, directionY, directionZ), radius_(radius){
	}

	/**
	 * RXgN^
	 * @param source lz
	 */
	inline explicit Capsule(const float* const source) :
		origin_(source[0], source[1], source[2]),
		direction_(source[3], source[4], source[5]), radius_(source[6]){
	}

	//--------------------------------------------------------------------------
	// l̐ݒ
	//--------------------------------------------------------------------------
	/**
	 * l̐ݒ
	 * @param origin ݒ肷錴_
	 * @param direction ݒ肷
	 * @param radius ݒ肷锼a
	 */
	inline void set(
		const Vector3& origin, const Vector3& direction, float radius){
		origin_ = origin;
		direction_ = direction;
		radius_ = radius;
	}

	/**
	 * l̐ݒ
	 * @param originX ݒ肷錴_X
	 * @param originY ݒ肷錴_Y
	 * @param originZ ݒ肷錴_Z
	 * @param directionX ݒ肷X
	 * @param directionY ݒ肷Y
	 * @param directionZ ݒ肷Z
	 * @param radius ݒ肷锼a
	 */
	inline void set(float originX, float originY, float originZ,
		float directionX, float directionY, float directionZ, float radius){
		origin_.set(originX, originY, originZ);
		direction_.set(directionX, directionY, directionZ);
		radius_ = radius;
	}

	/**
	 * l̐ݒ
	 * @param source ݒlz
	 */
	inline void set(const float* const source){
		origin_.set(source[0], source[1], source[2]);
		direction_.set(source[3], source[4], source[5]);
		radius_ = source[6];
	}

	//--------------------------------------------------------------------------
	/**
	 * _̐ݒ
	 * @param origin ݒ肷錴_
	 */
	inline void setOrigin(const Vector3& origin){ origin_ = origin; }

	/**
	 * ̐ݒ
	 * @param direction ݒ肷
	 */
	inline void setDirection(const Vector3& direction){
		direction_ = direction;
	}

	/**
	 * a̐ݒ
	 * @param radius ݒ肷锼a
	 */
	inline void setRadius(float radius){ radius_ = radius; }

	//--------------------------------------------------------------------------
	/**
	 * ʒu̐ݒ
	 * @param source \[Xʒu
	 * @param target ^[Qbgʒu
	 */
	inline void setPositions(const Vector3& source, const Vector3& target){
		origin_ = source;
		direction_ = (target - source);
	}

	//--------------------------------------------------------------------------
	// l̎擾
	//--------------------------------------------------------------------------
	/**
	 * _̎擾
	 * @return _
	 */
	inline const Vector3& getOrigin() const{ return origin_; }

	/**
	 * ̎擾
	 * @return 
	 */
	inline const Vector3& getDirection() const{ return direction_; }

	/**
	 * a̎擾
	 * @return a
	 */
	inline float getRadius() const{ return radius_; }

	//--------------------------------------------------------------------------
	/**
	 * \[Xʒu̎擾
	 * @return \[Xʒu
	 */
	inline const Vector3& getSourcePosition() const{ return origin_; }

	/**
	 * ^[Qbgʒu̎擾
	 * @return ^[Qbgʒu
	 */
	inline Vector3 getTargetPosition() const{ return (origin_ + direction_); }

	//--------------------------------------------------------------------------
	// JvZZ
	//--------------------------------------------------------------------------
	/**
	 * [JvZǂ
	 * @return [JvZȂtrue
	 */
	inline bool isZero() const{
		return (direction_.epsilonEquals(Vector3::zero, Math::epsilon) &&
			(Math::abs(radius_) <= Math::epsilon));
	}

	//--------------------------------------------------------------------------
	// gXtH[
	//--------------------------------------------------------------------------
	/**
	 * gXtH[
	 * @param matrix Zs
	 * @return ϊ̃JvZ
	 */
	inline Capsule transform(const Matrix33& matrix) const{
		return Capsule(matrix * origin_, matrix * direction_, radius_);
	}

	/**
	 * gXtH[
	 * @param matrix Zs
	 * @return ϊ̃JvZ
	 */
	inline Capsule transform(const Matrix34& matrix) const{
		return Capsule(matrix * origin_,
			matrix.multiply33(direction_), radius_);
	}

	/**
	 * gXtH[
	 * @param matrix Zs
	 * @return ϊ̃JvZ
	 */
	inline Capsule transform(const Matrix44& matrix) const{
		return Capsule(matrix * origin_,
			matrix.multiply33(direction_), radius_);
	}

	//--------------------------------------------------------------------------
	/**
	 * XP[LgXtH[
	 *
	 * ԃXP[̑傫Ȏɂ킹Ĕac݂܂B
	 * @param matrix Zs
	 * @return ϊ̃JvZ
	 */
	inline Capsule scaledTransform(const Matrix33& matrix) const{
		// xNg(1,0,0),(0,1,0),(0,0,1)ϊčŒ̒𔼌aɏZ
		float maxSquardLength = 0.f;
		for(int i = 0; i < 3; i++){
			float squaredLength =
				matrix.m[0][i] * matrix.m[0][i] +
				matrix.m[1][i] * matrix.m[1][i] +
				matrix.m[2][i] * matrix.m[2][i];
			if(squaredLength > maxSquardLength){
				maxSquardLength = squaredLength;
			}
		}
		return Capsule(matrix * origin_, matrix * direction_,
			radius_ * Math::sqrt(maxSquardLength));
	}

	/**
	 * XP[LgXtH[
	 *
	 * ԃXP[̑傫Ȏɂ킹Ĕac݂܂B
	 * @param matrix Zs
	 * @return ϊ̃JvZ
	 */
	inline Capsule scaledTransform(const Matrix34& matrix) const{
		// xNg(1,0,0),(0,1,0),(0,0,1)ϊčŒ̒𔼌aɏZ
		float maxSquardLength = 0.f;
		for(int i = 0; i < 3; i++){
			float squaredLength =
				matrix.m[0][i] * matrix.m[0][i] +
				matrix.m[1][i] * matrix.m[1][i] +
				matrix.m[2][i] * matrix.m[2][i];
			if(squaredLength > maxSquardLength){
				maxSquardLength = squaredLength;
			}
		}
		return Capsule(matrix * origin_, matrix.multiply33(direction_),
			radius_ * Math::sqrt(maxSquardLength));
	}

	/**
	 * XP[LgXtH[
	 *
	 * ԃXP[̑傫Ȏɂ킹Ĕac݂܂B
	 * @param matrix Zs
	 * @return ϊ̃JvZ
	 */
	inline Capsule scaledTransform(const Matrix44& matrix) const{
		// xNg(1,0,0),(0,1,0),(0,0,1)ϊčŒ̒𔼌aɏZ
		float maxSquardLength = 0.f;
		for(int i = 0; i < 3; i++){
			float squaredLength =
				matrix.m[0][i] * matrix.m[0][i] +
				matrix.m[1][i] * matrix.m[1][i] +
				matrix.m[2][i] * matrix.m[2][i];
			if(squaredLength > maxSquardLength){
				maxSquardLength = squaredLength;
			}
		}
		return Capsule(matrix * origin_, matrix.multiply33(direction_),
			radius_ * Math::sqrt(maxSquardLength));
	}

	//--------------------------------------------------------------------------
	// 
	//--------------------------------------------------------------------------
	/**
	 * _
	 * @param point 肷_
	 * @return 
	 */
	float getDistance(const Vector3& point) const{
		return Math::sqrt(getSquaredDistance(point));
	}

	/**
	 * _̓
	 * @param point 肷_
	 * @return ̓
	 */
	float getSquaredDistance(const Vector3& point) const;

	//--------------------------------------------------------------------------
	/**
	 * {bNX
	 * @param axisAlignedBox 肷鎲{bNX
	 * @return 
	 */
	float getDistance(const AxisAlignedBox& axisAlignedBox) const{
		return Math::sqrt(getSquaredDistance(axisAlignedBox));
	}

	/**
	 * {bNX̓
	 * @param axisAlignedBox 肷鎲{bNX
	 * @return ̓
	 */
	float getSquaredDistance(const AxisAlignedBox& axisAlignedBox) const;

	//--------------------------------------------------------------------------
	/**
	 * JvZ
	 * @param capsule 肷JvZ
	 * @return 
	 */
	float getDistance(const Capsule& capsule) const{
		return Math::sqrt(getSquaredDistance(capsule));
	}

	/**
	 * JvZ̓
	 * @param capsule 肷JvZ
	 * @return ̓
	 */
	float getSquaredDistance(const Capsule& capsule) const;

	//--------------------------------------------------------------------------
	/**
	 * R[
	 * @param cone 肷R[
	 * @return 
	 */
	float getDistance(const Cone& cone) const{
		return Math::sqrt(getSquaredDistance(cone));
	}

	/**
	 * R[̓
	 * @param cone 肷R[
	 * @return ̓
	 */
	float getSquaredDistance(const Cone& cone) const;

	//--------------------------------------------------------------------------
	/**
	 * C
	 * @param line 肷郉C
	 * @return 
	 */
	float getDistance(const Line& line) const{
		return Math::sqrt(getSquaredDistance(line));
	}

	/**
	 * C̓
	 * @param line 肷郉C
	 * @return ̓
	 */
	float getSquaredDistance(const Line& line) const;

	//--------------------------------------------------------------------------
	/**
	 * w{bNX
	 * @param orientedBox 肷w{bNX
	 * @return 
	 */
	float getDistance(const OrientedBox& orientedBox) const{
		return Math::sqrt(getSquaredDistance(orientedBox));
	}

	/**
	 * w{bNX̓
	 * @param orientedBox 肷w{bNX
	 * @return ̓
	 */
	float getSquaredDistance(const OrientedBox& orientedBox) const;

	//--------------------------------------------------------------------------
	/**
	 * ʋ
	 * @param plane 肷镽
	 * @return 
	 */
	float getDistance(const Plane& plane) const;

	/**
	 * ʋ̓
	 * @param plane 肷镽
	 * @return ̓
	 */
	float getSquaredDistance(const Plane& plane) const{
		float distance = getDistance(plane);
		return (distance * distance);
	}

	//--------------------------------------------------------------------------
	/**
	 * C
	 * @param ray 肷郌C
	 * @return 
	 */
	float getDistance(const Ray& ray) const{
		return Math::sqrt(getSquaredDistance(ray));
	}

	/**
	 * C̓
	 * @param ray 肷郌C
	 * @return ̓
	 */
	float getSquaredDistance(const Ray& ray) const;

	//--------------------------------------------------------------------------
	/**
	 * ZOg
	 * @param segment 肷ZOg
	 * @return 
	 */
	float getDistance(const Segment& segment) const{
		return Math::sqrt(getSquaredDistance(segment));
	}

	/**
	 * ZOg̓
	 * @param segment 肷ZOg
	 * @return ̓
	 */
	float getSquaredDistance(const Segment& segment) const;

	//--------------------------------------------------------------------------
	/**
	 * 
	 * @param sphere 肷鋅
	 * @return 
	 */
	float getDistance(const Sphere& sphere) const{
		return Math::sqrt(getSquaredDistance(sphere));
	}

	/**
	 * ̓
	 * @param sphere 肷鋅
	 * @return ̓
	 */
	float getSquaredDistance(const Sphere& sphere) const;

	//--------------------------------------------------------------------------
	/**
	 * Op
	 * @param triangle 肷Op
	 * @return 
	 */
	float getDistance(const Triangle& triangle) const{
		return Math::sqrt(getSquaredDistance(triangle));
	}

	/**
	 * Op̓
	 * @param triangle 肷Op
	 * @return ̓
	 */
	float getSquaredDistance(const Triangle& triangle) const;

	//--------------------------------------------------------------------------
	// 
	//--------------------------------------------------------------------------
	/**
	 * _
	 * @param point 肷_
	 * @return Ătrue
	 */
	bool intersect(const Vector3& point) const;

	//--------------------------------------------------------------------------
	/**
	 * {bNX
	 * @param axisAlignedBox 肷鎲{bNX
	 * @return Ătrue
	 */
	bool intersect(const AxisAlignedBox& axisAlignedBox) const;

	//--------------------------------------------------------------------------
	/**
	 * JvZ
	 * @param capsule 肷JvZ
	 * @return Ătrue
	 */
	bool intersect(const Capsule& capsule) const;

	//--------------------------------------------------------------------------
	/**
	 * R[
	 * @param cone 肷R[
	 * @return Ătrue
	 */
	bool intersect(const Cone& cone) const;

	//--------------------------------------------------------------------------
	/**
	 * C
	 * @param line 肷郉C
	 * @return Ătrue
	 */
	bool intersect(const Line& line) const;

	//--------------------------------------------------------------------------
	/**
	 * w{bNX
	 * @param orientedBox 肷w{bNX
	 * @return Ătrue
	 */
	bool intersect(const OrientedBox& orientedBox) const;

	//--------------------------------------------------------------------------
	/**
	 * ʌ
	 * @param plane 肷镽
	 * @return Ătrue
	 */
	bool intersect(const Plane& plane) const;

	//--------------------------------------------------------------------------
	/**
	 * C
	 * @param ray 肷郌C
	 * @return Ătrue
	 */
	bool intersect(const Ray& ray) const;

	//--------------------------------------------------------------------------
	/**
	 * ZOg
	 * @param segment 肷ZOg
	 * @return Ătrue
	 */
	bool intersect(const Segment& segment) const;

	//--------------------------------------------------------------------------
	/**
	 * 
	 * @param sphere 肷鋅
	 * @return Ătrue
	 */
	bool intersect(const Sphere& sphere) const;

	//--------------------------------------------------------------------------
	/**
	 * Op
	 * @param triangle 肷Op
	 * @return Ătrue
	 */
	bool intersect(const Triangle& triangle) const;

	//--------------------------------------------------------------------------
	// _Z
	//--------------------------------------------------------------------------
	/**
	 * JvZǂ
	 * @param target rJvZ
	 * @return lłtrueԂ
	 */
	inline bool operator ==(const Capsule& target) const{
		return ((origin_ == target.origin_) &&
			(direction_ == target.direction_) &&
			(radius_ == target.radius_));
	}

	/**
	 * JvZǂ
	 * @param target rJvZ
	 * @param epsilon 덷
	 * @return 덷͈͓̔œlłtrueԂ
	 */
	inline bool epsilonEquals(
		const Capsule& target, float epsilon) const{
		Assert(epsilon >= 0.f);
		return (origin_.epsilonEquals(target.origin_, epsilon) &&
			direction_.epsilonEquals(target.direction_, epsilon) &&
			(Math::abs(radius_ - target.radius_) <= epsilon));
	}

	/**
	 * JvZłȂǂ
	 * @param target rJvZ
	 * @return łȂlłtrueԂ
	 */
	inline bool operator !=(const Capsule& target) const{
		return ((origin_ != target.origin_) ||
			(direction_ != target.direction_) || (radius_ != target.radius_));
	}

	/**
	 * JvZłȂǂ
	 * @param target rJvZ
	 * @param epsilon 덷
	 * @return 덷͈͓̔œłȂlłtrueԂ
	 */
	inline bool notEpsilonEquals(
		const Capsule& target, float epsilon) const{
		Assert(epsilon >= 0.f);
		return (origin_.notEpsilonEquals(target.origin_, epsilon) ||
			direction_.notEpsilonEquals(target.direction_, epsilon) ||
			(Math::abs(radius_ - target.radius_) > epsilon));
	}

	//--------------------------------------------------------------------------
	// ̑
	//--------------------------------------------------------------------------
	/**
	 * 
	 * @return JvZ̕\L
	 */
	inline String toString() const{
		String returnString;
		returnString.format(
			"{ ( %.8f, %.8f, %.8f ) ( %.8f, %.8f, %.8f ) %.8f }",
			origin_.x, origin_.y, origin_.z,
			direction_.x, direction_.y, direction_.z, radius_);
		return returnString;
	}

private:
	//--------------------------------------------------------------------------
	// oϐ
	//--------------------------------------------------------------------------
	// _
	Vector3 origin_;
	// 
	Vector3 direction_;
	// a
	float radius_;

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of CAPSULE_H_
//------------------------------------------------------------------------------
